"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.remarkCodeBlock = void 0;
const unist_util_visit_1 = require("unist-util-visit");
const core_js_1 = require("refractor/lib/core.js");
const splitTextsByNewLine_js_1 = require("./splitTextsByNewLine.js");
const parse_numeric_range_1 = __importDefault(require("parse-numeric-range"));
const { register } = core_js_1.refractor;
const remarkCodeBlock = (options) => {
    const { enableLineNumbers = true } = options;
    const codeVisitor = (node) => {
        var _a;
        const languages = core_js_1.refractor.listLanguages();
        const { lang } = node;
        // no need to highlight since we have no idea what language is it
        if (!lang)
            return;
        if (!languages.includes(lang)) {
            // lang is not registered yet
            console.warn(`${lang} is not registered yet`);
            return;
        }
        node.data = (_a = node.data) !== null && _a !== void 0 ? _a : {};
        node.data.hProperties = node.data.hProperties || {};
        node.data.hProperties['data-lang'] = lang;
        let ast;
        let highlightedLines = [];
        // note that mast-util-to-hast will add `language-${lang}` to `code` per https://github.com/syntax-tree/mdast-util-to-hast/blob/7d20e86fbb0d8517497a0651dfcb229a6091a0c7/lib/handlers/code.js#L22
        // handle meta
        const { meta } = node;
        if (meta) {
            const params = new URLSearchParams(meta.split(/\s+/).join('&'));
            const filename = params.get('filename');
            if (filename) {
                ;
                node.data.hProperties['data-filename'] = filename;
            }
            // handle line highlight
            const lines = params.get('lines');
            if (lines) {
                if (enableLineNumbers === false) {
                    console.warn(`Please enable 'enableLineNumbers' option to use line highlighting feature`);
                }
                highlightedLines = parseLines(lines);
            }
        }
        if (enableLineNumbers) {
            ast = (0, splitTextsByNewLine_js_1.splitTextsByNewLine)(node.value)
                .map((lineText) => core_js_1.refractor.highlight(lineText, lang).children)
                .map((node, index) => ({
                type: 'element',
                tagName: 'span',
                children: node,
                properties: Object.assign({ dataLineNumber: index + 1 }, (highlightedLines.includes(index + 1)
                    ? {
                        dataLineHighlight: true,
                    }
                    : {})),
            }));
        }
        else {
            ast = core_js_1.refractor.highlight(node.value, lang).children;
        }
        // https://github.com/syntax-tree/mdast-util-to-hast/blob/a5c2f078fc219619eedd174ca857dd11e23abd4b/readme.md#fields-on-nodes
        node.data.hChildren = ast;
    };
    return (tree) => {
        (0, unist_util_visit_1.visit)(tree, 'code', codeVisitor);
    };
};
exports.remarkCodeBlock = remarkCodeBlock;
exports.remarkCodeBlock.register = (lang) => register(lang);
function parseLines(linesParam) {
    const lines = linesParam.match(/^\[(.+)\]$/);
    if (!lines) {
        return [];
    }
    else {
        return (0, parse_numeric_range_1.default)(lines[1]);
    }
}
